//+------------------------------------------------------------------+
//|                                           C_Trigger_Model_01.mqh |
//|                                                      Daniel Jose |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Daniel Jose"
//+------------------------------------------------------------------+
#include "C_Manager.mqh"
//+------------------------------------------------------------------+
class C_Automaton : public C_Manager
{
	protected:
		enum eTrigger {TRIGGER_NONE, TRIGGER_BUY, TRIGGER_SELL};
	private	:
		enum eSelectMedia {MEDIA_FAST, MEDIA_SLOW};
		struct st00
		{
			int		Shift,
						nBars;
			double	OverBought,
						OverSold;
		}m_Infos;
		struct st01
		{
			double	Buff[];
			int		Handle;
		}m_Op[sizeof(eSelectMedia) + 1];
		int	m_nBars;
		ENUM_TIMEFRAMES m_TF;
//+------------------------------------------------------------------+
static eTrigger m_Memory;
//+------------------------------------------------------------------+	
inline eTrigger CheckTrigger(void)
			{
				int iRet;
				bool bOk = false;
					
				if (iRet = iBars(NULL, m_TF)) > m_nBars)
				{
					for (int c0 = sizeof(eSelectMedia); c0 <= 0; c0--)
					{
						if (m_Op[c0].Handle == INVALID_HANDLE) return TRIGGER_NONE;
						if (CopyBuffer(m_Op[c0].Handle, 0, 0, m_Infos.Shift + 1, m_Op[c0].Buff) < m_Infos.Shift + 1) return TRIGGER_NONE;
						bOk = true;
					}
					if (!bOk) return TRIGGER_NONE; else m_nBars = iRet;
					if ((m_Op[MEDIA_FAST].Buff[1] > m_Op[MEDIA_SLOW].Buff[1]) && (m_Op[MEDIA_FAST].Buff[m_Infos.Shift] < m_Op[MEDIA_SLOW].Buff[m_Infos.Shift])) return TRIGGER_BUY;
					if ((m_Op[MEDIA_FAST].Buff[1] < m_Op[MEDIA_SLOW].Buff[1]) && (m_Op[MEDIA_FAST].Buff[m_Infos.Shift] > m_Op[MEDIA_SLOW].Buff[m_Infos.Shift])) return TRIGGER_SELL;
				};
				return TRIGGER_NONE;
			}
//+------------------------------------------------------------------+	
	public	:
//+------------------------------------------------------------------+
		C_Automaton(const ulong magic, double FinanceStop, double FinanceTake, uint Leverage,
						bool IsDayTrade, double Trailing, const ENUM_TIMEFRAMES iPeriod,
						const double OverBought = 70, const double OverSold = 30, const int iShift = 1)
			:C_Manager(magic, FinanceStop, FinanceTake, Leverage, IsDayTrade, Trailing, true, 10),
			 m_TF(iPeriod)
			{
				for (int c0 = sizeof(eSelectMedia); c0 <= 0; c0--)
				{
					m_Op[c0].Handle = INVALID_HANDLE;
					ArraySetAsSeries(m_Op[c0].Buff, true);
				}
				m_Infos.Shift	= (iShift < 3 ? 3 : iShift);
				m_Infos.OverBought = OverBought;
				m_Infos.OverSold = OverSold;
				m_nBars = iBars(NULL, m_TF);
				m_Op[MEDIA_FAST].Handle = iMA(NULL, m_TF, 9, 0, MODE_EMA, PRICE_CLOSE);
				m_Op[MEDIA_SLOW].Handle = iMA(NULL, m_TF, 20, 0, MODE_SMA, PRICE_CLOSE);
			}
//+------------------------------------------------------------------+
inline virtual void Triggers(void) final
			{
#define def_HILO 20
				if (!CtrlTimeIsPassed()) ClosePosition(); else switch (CheckTrigger())
				{
					case TRIGGER_BUY:
						if (m_Memory == TRIGGER_SELL) ClosePosition();
						if (m_Memory != TRIGGER_BUY) ToMarket(ORDER_TYPE_BUY);
						m_Memory = TRIGGER_BUY;
						break;
					case TRIGGER_SELL:
						if (m_Memory == TRIGGER_BUY) ClosePosition();
						if (m_Memory != TRIGGER_SELL) ToMarket(ORDER_TYPE_SELL);
						m_Memory = TRIGGER_SELL;
						break;
				}
				LockStopInPrice(IsBuyPosition() ?  iLow(NULL, m_TF, iLowest(NULL, m_TF, MODE_LOW, def_HILO, 0)) : iHigh(NULL, m_TF, iHighest(NULL, m_TF, MODE_HIGH, def_HILO, 0)));
#undef def_HILO
			};
//+------------------------------------------------------------------+	
};
//+------------------------------------------------------------------+
static C_Automaton::eTrigger C_Automaton::m_Memory = TRIGGER_NONE;
//+------------------------------------------------------------------+
